// BitmapWindow.cpp: implementation of the CBitmapWindow class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Vectal4.h"
#include "BitmapWindow.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBitmapWindow::CBitmapWindow()
{
    m_nWindowW = 0;
    m_nWindowH = 0;

    m_bReInit  = 1;
}

CBitmapWindow::~CBitmapWindow()
{
    Release();
}




void CBitmapWindow::ReInit( CDC* pDC )
{
    int nW;
    int nH;

    if( 1 == m_bReInit )  {     // Do not Recreate Memory-Bitmap for Each Resizing
        nW = GetSystemMetrics(SM_CXSCREEN);
        nH = GetSystemMetrics(SM_CYSCREEN);
    }
    else  {                     // Exact Size (for SaveAsBmp)
        nW = m_nWindowW;
        nH = m_nWindowH;
    }


    Release();
    m_cb.CreateCompatibleBitmap( pDC, nW,nH );

    m_bReInit = 0;
}



void CBitmapWindow::Release()
{
    // Delete GDI Objects
    if( (HBITMAP)m_cb )
        m_cb.DeleteObject();
}




void CBitmapWindow::BeginPaint( CDC* pDC )
{
    // Once Initialize
    if( m_bReInit )
        ReInit( pDC );

    // Select MemBitmap to MemDC
    m_dcb.CreateCompatibleDC( pDC );
    m_pcbOld = m_dcb.SelectObject( &m_cb );

}

void CBitmapWindow::EndPaint( CDC* pDC )
{
    // Blit MemBitmap and Delete MemDC

    if( pDC )
        pDC->BitBlt( 0,0,m_nWindowW,m_nWindowH, &m_dcb, 0,0, SRCCOPY );

    m_dcb.SelectObject( m_pcbOld );
    m_dcb.DeleteDC();
}





BOOL CBitmapWindow::SaveAsBmp( LPCTSTR szFileName )
{
    BOOL bRes = 0;

    // Det Color Bits
    BITMAP bmp;
    m_cb.GetBitmap( &bmp );
    WORD cClrBits = bmp.bmBitsPixel;
    if( cClrBits < 16 )
        return 0;   // Without Indexed Colors, Please


    // BITMAPINFOHEADER
    BITMAPINFO* pbmi = (PBITMAPINFO)LocalAlloc( LPTR, sizeof(BITMAPINFOHEADER) );


    BITMAPINFOHEADER* pbih = &pbmi->bmiHeader;
    pbih->biSize         = sizeof(BITMAPINFOHEADER);
    pbih->biWidth        = m_nWindowW;
    pbih->biHeight       = m_nWindowH;
    pbih->biPlanes       = 1;
    pbih->biBitCount     = cClrBits;
    pbih->biCompression  = BI_RGB; 
    pbih->biSizeImage    = (((m_nWindowW * cClrBits + 31) & ~31) >> 3) * m_nWindowH;
    pbih->biClrImportant = 0; 


    // BITS..
    LPBYTE lpBits = (LPBYTE)GlobalAlloc( GMEM_FIXED, pbih->biSizeImage );
    m_cb.GetBitmapBits( pbih->biSizeImage, lpBits );


    // FILE
    CFile file;
    if( file.Open( szFileName, CFile::modeCreate | CFile::modeWrite ) )
    {
        BITMAPFILEHEADER hdr = {0};
        hdr.bfType = 0x4d42;        // 0x42 = "B" 0x4d = "M" 
        hdr.bfSize = (DWORD)( sizeof(BITMAPFILEHEADER) 
                              + pbih->biSize
                              + pbih->biClrUsed * sizeof(RGBQUAD) 
                              + pbih->biSizeImage );

        // Compute the offset to the array of color indices. 
        hdr.bfOffBits = (DWORD)sizeof(BITMAPFILEHEADER) 
                        + pbih->biSize
                        + pbih->biClrUsed * sizeof(RGBQUAD);

        // Copy the BITMAPFILEHEADER into the .BMP file. 
        file.Write( &hdr, sizeof(BITMAPFILEHEADER) );

        // Copy the BITMAPINFOHEADER and RGBQUAD array into the file. 
        file.Write( pbih, sizeof(BITMAPINFOHEADER) + pbih->biClrUsed * sizeof(RGBQUAD) );

        // Copy the array of color indices into the .BMP file. 
        file.Write( lpBits, pbih->biSizeImage );
    
        file.Close();

        bRes = 1;
    }

    
    GlobalFree( lpBits );

    GlobalFree( pbmi );
 
    return bRes;
}
