// Draw.cpp: implementation of the CDraw class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Vectal4.h"
#include "Draw.h"

#include <math.h>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDraw::CDraw()
{

    m_bRedraw = 1;

    m_bShowPattern = 1;
    m_bShowFractal = 1;
    m_bFillMode    = 0;


    // Create Pens
    m_pens[0].CreatePen( PS_SOLID, 1, (COLORREF)0xCFDFCF );
    m_pens[1].CreatePen( PS_DASH,  1, (COLORREF)0xCFDFCF );
    m_pens[2].CreatePen( PS_SOLID, 1, (COLORREF)0xCFCFDF );
    m_pens[3].CreatePen( PS_DASH,  1, (COLORREF)0xCFCFDF );
    m_pens[4].CreatePen( PS_DOT,   1, (COLORREF)0xEFEFEF );
    m_pens[5].CreatePen( PS_SOLID, 1, (COLORREF)0x000000 );


    // Iterations
    m_nIterations = 7;
    m_nTimeLimit  = 1000;
    m_nDetail     = 2;


    m_nFillBuffer  = 100000;
    m_ptFillBuffer = (CPoint*)GlobalAlloc( GMEM_FIXED, (m_nFillBuffer+5) * sizeof(CPoint) );


    // Economy
    for( int i=0; i<DIST; i++ )
     for( int j=0; j<DIST; j++ )
        m_nDists[i][j] = int( sqrt(i*i+j*j) );

    m_bEconomy = 1;


    m_bDrawToAI = 0;

}

CDraw::~CDraw()
{
    //
    if( m_ptFillBuffer )
        GlobalFree( m_ptFillBuffer );


    // Delete GDI Objects
    for( int i=0; i<PENS; i++ )
        if( (HPEN)m_pens[i] )
            m_pens[i].DeleteObject(); 

}


//////////////////////////////////////////////////////////////////////

void CDraw::OnSize( int nW, int nH )
{
    if( CBitmapWindow::m_nWindowW ^ nW  ||
        CBitmapWindow::m_nWindowH ^ nH  )
    {
        CBitmapWindow::m_nWindowW = nW;
        CBitmapWindow::m_nWindowH = nH;

        CSpacer::SetOffset( CPoint( nW>>1, nH>>1 ) );

        // Set Bound
        m_rtBound.SetRect( 0,0, nW,nH );


        m_bRedraw = 1;
    }    
}


//////////////////////////////////////////////////////////////////////


void CDraw::DrawVectal( CPattern* pPattern )
{
    // Get Pattern Data
    m_nKneesN        = pPattern->GetKneesN();
    CPoinf* pfPoinfs = pPattern->GetPoinfs();
    BYTE*   bDirs    = pPattern->GetDirs();



    // Erase background
    m_dcb.FillSolidRect( 0, 0, m_nWindowW, m_nWindowH, 0xFFFFFF );

    
    // Draw Pattern
    if( m_bShowPattern )
    {
        CPoint p;
        int i;

/*
        // Draw Head
        p = CSpacer::ToPoint( pfPoinfs[0] );

        CSize  r(3,3);
        CRect  rect(p-r,p+r);
        m_dcb.SelectObject( &m_pens[0] );
        m_dcb.Rectangle( &rect );

        m_dcb.MoveTo( p );
*/

        // Draw Points

        m_dcb.SelectObject( &m_pens[0] );

        CSize  r(3,3);
        CRect  rect;
        for( i=0; i<=m_nKneesN; i++ )
        {
            p = CSpacer::ToPoint( pfPoinfs[i] );
            rect.SetRect( p-r,p+r );
            m_dcb.Rectangle( &rect );
        }

    

        // Draw Knees

        p = CSpacer::ToPoint( pfPoinfs[0] );
        m_dcb.MoveTo( p );

        for( i=1; i<=m_nKneesN; i++ )
        {   
            BYTE bDir = bDirs[i-1];

            if( DIR_HIDE & bDir )
                m_dcb.SelectObject( &m_pens[ 4 ] );
            else if( DIR_STATIC & bDir )
                m_dcb.SelectObject( &m_pens[ 5 ] );
            else
                m_dcb.SelectObject( &m_pens[ bDir & 3 ] );
            
            p = CSpacer::ToPoint( pfPoinfs[i] );
            m_dcb.LineTo( p );
        }

    }

    // Draw Vectal
    if( m_bShowFractal )
    {
        if( m_bFillMode )
        {
            m_nPointCur = 0;
            m_dcb.SelectStockObject( GRAY_BRUSH );
        }

        m_dcb.SelectObject( &m_pens[5] );

        DrawPolyline( pfPoinfs, bDirs );

        if( m_bFillMode )
        {
            CloseOutPolygon( pfPoinfs );
            
            m_dcb.Polygon( m_ptFillBuffer, m_nPointCur );
        }

    }


    m_bRedraw = 0;
}


void CDraw::DrawPolyline( CPoinf* pfPoinfs, BYTE* bDirs )
{
    int i;


    // Det 2-Sided Dirs
    for( i=0; i<m_nKneesN; i++)
    {        
        m_bDirs[0][i] = bDirs[i];
        m_bDirs[1][i] = bDirs[m_nKneesN-1-i];
    }

    // Get First,Last Points
    CPoinf f[2];
    f[0] = pfPoinfs[    0    ];
    f[1] = pfPoinfs[m_nKneesN];

        
    // Det 2-Sided Normalized Sample
    CPoinf p = f[1] - f[0];
    p.ToPolar();
    
    for( i=1; i<m_nKneesN; i++)
    {        
        CPoinf pfI;

        pfI = pfPoinfs[i] - f[0];
        pfI.ToPolar();

        m_pfNormas[0][i].r = pfI.r / p.r;
        m_pfNormas[0][i].a = pfI.a - p.a;

        pfI = f[1] - pfPoinfs[i];
        pfI.ToPolar();

        m_pfNormas[1][m_nKneesN-i].r = pfI.r / p.r;
        m_pfNormas[1][m_nKneesN-i].a = pfI.a - p.a;
    }


    // Add First Point
    if( m_bDrawToAI )
        AIAddPoint( f[0], AP_FIRST );
    else  {
        CPoint p0 = CSpacer::ToPoint( f[0] );
        AddPoint( p0, m_rtBound.PtInRect(p0), AP_FIRST );
    }

    // Prepare Time-Limit
    m_dwTickLimit = GetTickCount() + m_nTimeLimit;
    m_nIterationCalls = 0;


    // Make Iterations
    DrawPolylineIteration( f[0], f[1], 0, m_nIterations );
    
}


// Return 0 if Error
BYTE CDraw::AddPoint( const CPoint& pt, BYTE bInBound, BYTE bMode )
{   
    if( m_bFillMode )
    {
        if( m_nPointCur < m_nFillBuffer  &&
            ( !m_bEconomy  ||  m_nPointCur < m_nTimeLimit<<3 ) )
            m_ptFillBuffer[ m_nPointCur++ ] = pt;
        else
            return 0;
    }
    else
    {
        // Move or Draw Line
        if( AP_MOVE & bMode )  m_dcb.MoveTo( pt );
        else                   m_dcb.LineTo( pt );
    }    

    // Save Last Point
    m_ptLast       = pt;
    m_bLastInBound = bInBound;

    return 1;
}


// Return 0 if Error
BYTE CDraw::DrawPolylineIteration( const CPoinf& f0,
                                   const CPoinf& f1, BYTE bDir, BYTE nIteration )
{   
    //----------------------------------- Check Limitations
    
    if( m_bDrawToAI )       // Work with PoinF Only
    {
        if( DIR_HIDE & bDir )  {
            AIAddPoint( f1, AP_MOVE );
            return 1;
        }

        if( DIR_STATIC & bDir )  {
            AIAddPoint( f1, AP_LINE );
            return 1;
        }

        if( !nIteration )  {  // No more iterations - Draw line!
            AIAddPoint( f1, AP_LINE );
            return 1;
        }

    }
    else    // No Fill Mode - Draw Lines
    {
        CPoint p1 = CSpacer::ToPoint( f1 );

        BYTE bInBound = m_rtBound.PtInRect( p1 );

        if( DIR_HIDE & bDir )
            return AddPoint( p1, bInBound, AP_MOVE );

        if( DIR_STATIC & bDir )
            return AddPoint( p1, bInBound, AP_LINE );

        LONG nD = GetDist( CPoint(p1 - m_ptLast) );

        if( !nIteration  ||     // No more iterations - Draw!
            nD < m_nDetail  )   // Do not Iterate Small Lines
        {
            if( !m_bEconomy )
                return AddPoint( p1, bInBound, AP_LINE );

            else if( m_bLastInBound | bInBound )
            {
                m_nIterationCalls ++;
                if( !(m_nIterationCalls & 0xFF) )         // Check each 0xFFth call only
                    if( m_dwTickLimit < GetTickCount() )
                        return 0;

                return AddPoint( p1, bInBound, AP_LINE );
            }
            else
                return AddPoint( p1, bInBound, AP_MOVE );
        }

    }        

    //--------------------------------- Do Iteration

    nIteration --;

    //
    CPoinf p = f1;
    p -= f0;
    p.ToPolar();         // For getNF1

    // f0  .   .   .   f1
    // nf0 nf1
    //     nf0 nf1
    //         nf0 nf1

    CPoinf nf0;
    CPoinf nf1 = f0;
        
    for( int i=1; i<m_nKneesN; i++ )
    {
        nf0 = nf1;
        nf1 = GetNF1( f0,p, i, bDir );
        
        if( !DrawPolylineIteration( nf0,nf1, bDir ^ m_bDirs[ DIR_REVERSE & bDir ][i-1], nIteration ) )
            return 0;
    }

    // Last point is known
    return DrawPolylineIteration( nf1,f1, bDir ^ m_bDirs[ DIR_REVERSE & bDir ][i-1], nIteration );
        
}


CPoinf CDraw::GetNF1( const CPoinf &f0, const CPoinf &p, BYTE i, BYTE bDir )
{
    CPoinf& fk = m_pfNormas[ DIR_REVERSE & bDir ][i];
    
    float r = p.r * fk.r;
    float a = p.a + ( DIR_MIRROR & bDir? -fk.a : fk.a );

    float x = r * cosf(a) + f0.x;
    float y = r * sinf(a) + f0.y;
 
    return CPoinf(x,y);
}


//////////////////////////////////////////////////////////////////////
// Fill Mode

void CDraw::CloseOutPolygon( CPoinf* pfPoinfs )
{
    int i;

    // Get First,Last Points
    CPoinf f0 = pfPoinfs[    0    ];
    CPoinf f1 = pfPoinfs[m_nKneesN];
    CPoinf fd = f1 - f0;

    // Bounding Rect
    const int bar = 2;
    CPoinf br[4];
    br[0] = CSpacer::ToPoinf( CPoint( CBitmapWindow::m_nWindowW+bar,                          -bar) );
    br[1] = CSpacer::ToPoinf( CPoint(                          -bar,                          -bar) );
    br[2] = CSpacer::ToPoinf( CPoint(                          -bar, CBitmapWindow::m_nWindowH+bar) );
    br[3] = CSpacer::ToPoinf( CPoint( CBitmapWindow::m_nWindowW+bar, CBitmapWindow::m_nWindowH+bar) );

    CPoinf bd[4];
    for( i=0; i<4; i++ )
        bd[i] = br[i] - f1;

    // Cross Sides
    BYTE cs[2];
    BYTE csn = 0;

    float leftp;
    float leftc = bd[3].y * fd.x - bd[3].x * fd.y;

    for( i=0; i<4; i++ )
    {
        leftp = leftc;
        leftc = bd[i].y * fd.x - bd[i].x * fd.y;

        if( leftp * leftc < 0.0f )
           cs[ csn++ ] = i;
    }

    if( csn < 2 )  // No Cross Sides Found
        return;

    // Cross Poinfs
    CPoinf cf[2];
     
    for( i=0; i<2; i++ )
    {
        BYTE csi = cs[i];

        if( csi & 1 )
        {
            cf[i].y = br[csi].y;

            if( 0 == fd.x ) cf[i].x = f1.x;
            else            cf[i].x = (f1.x * fd.y - fd.x * (f1.y - br[csi].y) ) / fd.y;
        }
        else
        {
            cf[i].x = br[csi].x;

            if( 0 == fd.y ) cf[i].y = f1.y;
            else            cf[i].y = (f1.y * fd.x - fd.y * (f1.x - br[csi].x) ) / fd.x;
        }
    }    

    // Poinfs Order
    CPoinf cd[2];
    for( i=0; i<2; i++ )
        cd[i] = cf[i] - f1;

    BYTE fo = fd.x * cd[0].x + fd.y * cd[0].y <
              fd.x * cd[1].x + fd.y * cd[1].y;


    // Add Points
    BYTE s = cs[ fo ];

    if( m_bDrawToAI )
    {
        AIAddPoint( cf[ fo ], AP_LINE );

        do  {
            s += 3;
            s &= 3;
            AIAddPoint( br[ s ], AP_LINE );
        }
        while( s != cs[fo^1] );

        AIAddPoint( cf[fo^1], AP_LINE );
    }
    else
    {    
        CPoint pt;

        pt = CSpacer::ToPoint( cf[ fo ] );
        AddPoint( pt, 0, AP_LINE );

        do  {
            s += 3;
            s &= 3;
            pt = CSpacer::ToPoint( br[ s ] );
            AddPoint( pt, 0, AP_LINE );
        }
        while( s != cs[fo^1] );

        pt = CSpacer::ToPoint( cf[fo^1] );
        AddPoint( pt, 0, AP_LINE );
    }    
}



//////////////////////////////////////////////////////////////////////
// AI Writing


BOOL CDraw::DrawToAI( CPattern* pPattern, LPCTSTR szFileName )
{
    // Saving
    if( !CAIWriter::m_file.Open( szFileName, CFile::modeCreate | 
                                             CFile::modeWrite ) )
        return 0;

    m_bDrawToAI = 1;


    // Get Pattern Data
    m_nKneesN        = pPattern->GetKneesN();
    CPoinf* pfPoinfs = pPattern->GetPoinfs();
    BYTE*   bDirs    = pPattern->GetDirs();
    

    // Head
    CAIWriter::WriteHead();

    // Bounding Box
    CPoinf lb = CSpacer::ToPoinf( CPoint( 0, CBitmapWindow::m_nWindowH ) );
    CPoinf rt = CSpacer::ToPoinf( CPoint( CBitmapWindow::m_nWindowW, 0 ) );
    CAIWriter::WriteBoxes( lb.x, lb.y, rt.x, rt.y );

    // Prolog & Setup
    CAIWriter::WriteString( IDS_AI_PROLOG );
    CAIWriter::WriteString( IDS_AI_SETUP );


    // Draw Sample
    if( m_bShowPattern )
    {
        CAIWriter::WriteString( IDS_AI_BEGINLAYER );
        CAIWriter::WriteString( IDS_AI_LAYERPATTERN );

        // Draw Knees
        for( int i=1; i<=m_nKneesN; i++ )
        {   
            CAIWriter::WriteString( IDS_AI_SETSTROKE );

            if( DIR_HIDE & bDirs[i-1] )
            {
                CAIWriter::WriteString( IDS_AI_PEN4 );
                CAIWriter::WriteString( IDS_AI_LINEWIDTH );
                CAIWriter::WriteString( IDS_AI_PENDOT );
            }
            else
            {
                CAIWriter::WriteString( DIR_REVERSE & bDirs[i-1] ? IDS_AI_PEN2 : 
                                                                   IDS_AI_PEN0 );
                CAIWriter::WriteString( IDS_AI_LINEWIDTH );

                CAIWriter::WriteString( DIR_MIRROR  & bDirs[i-1] ? IDS_AI_PENDASH :
                                                                   IDS_AI_PENSOLID );
            }

            CAIWriter::WritePoinf( pfPoinfs[i-1].x, pfPoinfs[i-1].y, AP_MOVE );
            CAIWriter::WritePoinf( pfPoinfs[ i ].x, pfPoinfs[ i ].y, AP_LINE );
            CAIWriter::WriteString( "\r\nS" );
        }

        //
        CAIWriter::WriteString( IDS_AI_ENDLAYER );
    }


    // Draw Vectal
    if( m_bShowFractal )
    {
        CAIWriter::WriteString( IDS_AI_BEGINLAYER );
        CAIWriter::WriteString( IDS_AI_LAYERFRACTAL );
        //

        if( m_bFillMode )
        {   
            CAIWriter::WriteString( IDS_AI_SETFILL );
            CAIWriter::WriteString( IDS_AI_BRUSH );
        }

        CAIWriter::WriteString( IDS_AI_SETSTROKE );
        CAIWriter::WriteString( IDS_AI_PEN5 );
        CAIWriter::WriteString( IDS_AI_LINEWIDTH );
        CAIWriter::WriteString( IDS_AI_PENSOLID );

        //
        DrawPolyline( pfPoinfs, bDirs );
        //

        if( m_bFillMode )
        {   
            CloseOutPolygon( pfPoinfs );
            CAIWriter::WriteString( IDS_AI_CLOSEFILLSTROKE );
        }
        else
        {
            CAIWriter::WriteString( IDS_AI_STROKE );
        }

        //
        CAIWriter::WriteString( IDS_AI_ENDLAYER );
    }


    // End
    CAIWriter::WriteString( IDS_AI_EOF );

    //
    CAIWriter::m_file.Close();
    m_bDrawToAI = 0;

    return 1;
}



void CDraw::AIAddPoint( const CPoinf& pf, BYTE bMode )
{
    if( m_bFillMode )
        switch( bMode )  {
            case AP_FIRST:      CAIWriter::WritePoinf( pf.x, pf.y, AP_MOVE );
                                return;
            case AP_MOVE:
            case AP_LINE:       CAIWriter::WritePoinf( pf.x, pf.y, AP_LINE );
                                return;
        }
    else
        switch( bMode )  {
            case AP_MOVE:       CAIWriter::WriteString( IDS_AI_STROKE );
            case AP_FIRST:      CAIWriter::WritePoinf( pf.x, pf.y, AP_MOVE );
                                return;

            case AP_LINE:       CAIWriter::WritePoinf( pf.x, pf.y, AP_LINE );
                                return;
        }

}


