// Draw.h: interface for the CDraw class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DRAW_H__BC67F7C0_7693_40CF_85DE_90760F91E143__INCLUDED_)
#define      AFX_DRAW_H__BC67F7C0_7693_40CF_85DE_90760F91E143__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000



#include "BitmapWindow.h"
#include "Mouser.h"
#include "AIWriter.h"


#include "Pattern.h"


//////////////////////////////////////////////////////////////////////

#define TIMELIMITMAX    60000
#define DETAILMAX       100
#define ITERATIONSMAX   100

#define PENS 6


// AddPoint Modes
#define AP_LINE  0  // 00
#define AP_MOVE  1  // 01
#define AP_FIRST 3  // 11


#define DIST    1000


class CDraw : protected CBitmapWindow,
              protected CMouser,
              protected CAIWriter
{

private:

    // Economy
    LONG        m_nDists[DIST][DIST];
    LONG        GetDist( CPoint& pt )  {
                    LONG x = abs( pt.x );
                    LONG y = abs( pt.y );
                    if( x < DIST  &&  y < DIST )
                          return  m_nDists[x][y];
                    else  return  x + y;
                }

    CPoint      m_ptLast;

    CRect       m_rtBound;
    BYTE        m_bLastInBound;
    

    // Time Limit
    DWORD       m_dwTickLimit;              // = GetTickCount(Start) + TimeLimit
    int         m_nIterationCalls;          // Do not call GetTickCount() each Iteration


    // Pattern Data for Drawing
    int         m_nKneesN;
    BYTE        m_bDirs   [2][MAXKNEES];    // 2-Sided Dir
    CPoinf      m_pfNormas[2][MAXKNEES];    // 2-Sided Normalized Sample


    void        DrawPolyline( CPoinf* pfPoinfs, BYTE* bDirs );
    BYTE        DrawPolylineIteration( const CPoinf&, const CPoinf&, BYTE bDir, BYTE nIteration );
    BYTE        AddPoint( const CPoint&, BYTE bInBound, BYTE bMode );
    CPoinf      GetNF1( const CPoinf &f0, const CPoinf &p, BYTE i, BYTE bDir );

    // Fill Mode
    CPoint*     m_ptFillBuffer;
    int         m_nFillBuffer;
    int         m_nPointCur;

    void        CloseOutPolygon( CPoinf* pfPoinfs );


    // GDI
    CPen        m_pens[PENS];

    // AI Writing
    BYTE        m_bDrawToAI;
    void        AIAddPoint( const CPoinf& pf, BYTE bMode );


protected:

    // User Interface
    int         m_nIterations;
    int         m_nDetail;
    int         m_nTimeLimit;

    BYTE        m_bShowPattern;     // Access in CVectal4View::WM_COMMAND
    BYTE        m_bShowFractal;
    BYTE        m_bFillMode;



    BYTE        m_bRedraw;

    BYTE        m_bEconomy;         // Do not Economy in Export case



    void        OnSize( int, int );
    void        DrawVectal( CPattern* );
    BOOL        DrawToAI( CPattern*, LPCTSTR szFileName );




public:
	CDraw();
	virtual ~CDraw();

};



#endif // !defined(AFX_DRAW_H__BC67F7C0_7693_40CF_85DE_90760F91E143__INCLUDED_)
