// Mouser.cpp: implementation of the CMouser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Vectal4.h"
#include "Mouser.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMouser::CMouser()
{
    m_hArrows[0] = AfxGetApp()->LoadStandardCursor( IDC_ARROW   );
    m_hArrows[1] = AfxGetApp()->LoadCursor        ( IDC_ARROWP  );
    m_hArrows[2] = AfxGetApp()->LoadCursor        ( IDC_ARROWM  );
    m_hArrows[3] = AfxGetApp()->LoadStandardCursor( IDC_SIZEALL );
    m_hArrows[4] = AfxGetApp()->LoadCursor        ( IDC_ARROWD  );
    m_bArrow = 0;

    m_bCtrl = 0;

    m_bPopupMenuLoop = 0;

    m_menu.LoadMenu( IDR_MENU_MOUSER );
}

CMouser::~CMouser()
{
    m_menu.DestroyMenu();
}



void CMouser::TraceCursor( CPattern* pPattern )
{
    m_nElem = pPattern->DetCursorMode( m_pfCursor, CSpacer::GetScale(), m_bMode );
}

void CMouser::DetArrow()
{
    if( m_bCtrl )
        m_bArrow = MODESELPOINT==m_bMode? 2:
                                          1;
    else
        m_bArrow = MODESELPOINT==m_bMode? 3:
                   MODESELKNEE ==m_bMode? 4:
                                          0;
}



DWORD CMouser::OnMouseViewEvent( UINT message, WPARAM wParam )
{
    // Mouse Wheel
    if( WM_MOUSEWHEEL == message )
    {
        short zDelta = (short)HIWORD(wParam);
        float d = 1.0f * zDelta/WHEEL_DELTA;    // Delta

        CSpacer::Scale( m_pfCursor, d );
        return ME_PROCESSED | ME_REDRAW;
    }

    // Button Down or Mouse Move
    switch( wParam )
    {
        case MK_MBUTTON | MK_BUTTONDOWN:
            CSpacer::MoveOrigin( m_ptCursor, 1 );   // Start Move
            return ME_PROCESSED;

        case MK_MBUTTON:
            CSpacer::MoveOrigin( m_ptCursor, 0 );   // Move
            return ME_PROCESSED | ME_REDRAW;
        
        case ID_VIEW_RESTOREDEFAULTVIEW:
        case MK_MBUTTON | MK_BUTTONDOWN | MK_SHIFT:
            CSpacer::Restore();                     // Restore default
            return ME_PROCESSED | ME_REDRAW;
    }

    return 0;
}



DWORD CMouser::OnMouseDocEvent( UINT message, WPARAM wParam, CPattern* pPattern )
{
    // Popup Menu
    if( WM_COMMAND == message )
    {
        switch( LOWORD(wParam) )
        {
            case ID_MOUSER_REVERSE:
                pPattern->ChangeDir( m_nElem, DIR_REVERSE );
                return  ME_PROCESSED | ME_REDRAW;

            case ID_MOUSER_MIRROR:
                pPattern->ChangeDir( m_nElem, DIR_MIRROR );
                return  ME_PROCESSED | ME_REDRAW;

            case ID_MOUSER_HIDE:
                pPattern->ChangeDir( m_nElem, DIR_HIDE );
                return  ME_PROCESSED | ME_REDRAW;

            case ID_MOUSER_STATIC:
                pPattern->ChangeDir( m_nElem, DIR_STATIC );
                return  ME_PROCESSED | ME_REDRAW;

            case ID_MOUSER_REMOVE:
                if( -1 == pPattern->RemovePoint( m_nElem ) )
                    return ME_PROCESSED;
                return ME_PROCESSED | ME_REDRAW;

            case ID_MOUSER_ADDPOINT:
                if( -1 == pPattern->AddPoint( m_nElem, m_pfCursor ) )
                    return ME_PROCESSED;
                return ME_PROCESSED | ME_REDRAW;

            //
            case ID_VIEW_SETASDEFAULTVIEW:
                CSpacer::SetViewAsDefault( pPattern->GetPoinfs(),
                                           pPattern->GetKneesN() );
                return ME_PROCESSED | ME_REDRAW;

                
            case ID_POINT_COORDINATE:

//                CPoinf* pfPoinfs = pPattern->GetPoinfs();
                //m_dlgCoordinate.SetPoinf( pPattern->GetPoinfs() + m_nElem );
                m_dlgCoordinate.SetPoinf( pPattern, m_nElem );

                if( IDOK != m_dlgCoordinate.DoModal() )
                    return ME_PROCESSED;
                return ME_PROCESSED | ME_REDRAW;

        }

        return ME_PROCESSED;
    }


    // Button Down or Mouse Move: m_nElem found in TraceCursor()
    switch( wParam )
    {
        // Move Points
        case MK_LBUTTON | MK_BUTTONDOWN:    
        case MK_LBUTTON | MK_BUTTONDOWN | MK_SHIFT:

            if( MODESELKNEE == m_bMode )  // No Move
            {
                pPattern->ChangeDir( m_nElem, MK_SHIFT & wParam ? DIR_HIDE : 0 );

                return  ME_PROCESSED | ME_REDRAW;
            }

        case MK_LBUTTON:
//        case MK_LBUTTON | MK_SHIFT:   Do not move if Shift (Shift is just for Knee Hiding)

            if( MODESELPOINT == m_bMode )
            {
                pPattern->MovePoint( m_nElem, m_pfCursor );

                return  ME_PROCESSED | ME_REDRAW;
            }
            
            return ME_PROCESSED;


        // Add/Remove Point
        case MK_LBUTTON | MK_BUTTONDOWN | MK_CONTROL:

            if( MODESELPOINT == m_bMode )  // Near Point - Remove
            {
                if( -1 == pPattern->RemovePoint( m_nElem ) )
                    return ME_PROCESSED;
            }            
            else                            // Add
            {
                if( -1 == pPattern->AddPoint( m_nElem, m_pfCursor ) )
                    return ME_PROCESSED;
            }

            return ME_PROCESSED | ME_REDRAW;



        // Show Popup Menu
        case MK_RBUTTON | MK_BUTTONDOWN:
            
            CMenu* pMenu = m_menu.GetSubMenu( m_bMode );

            if( MODESELKNEE == m_bMode )  {
                BYTE bDir = pPattern->GetDir( m_nElem );
                pMenu->CheckMenuItem( ID_MOUSER_REVERSE, DIR_REVERSE & bDir ? MF_CHECKED : MF_UNCHECKED );
                pMenu->CheckMenuItem( ID_MOUSER_MIRROR,  DIR_MIRROR  & bDir ? MF_CHECKED : MF_UNCHECKED );
                pMenu->CheckMenuItem( ID_MOUSER_HIDE,    DIR_HIDE    & bDir ? MF_CHECKED : MF_UNCHECKED );
                pMenu->CheckMenuItem( ID_MOUSER_STATIC,  DIR_STATIC  & bDir ? MF_CHECKED : MF_UNCHECKED );
            }

            m_bPopupMenuLoop = 1;

            m_pWndParent->ClientToScreen( &m_ptCursor );
            pMenu->TrackPopupMenu( TPM_LEFTALIGN | TPM_RIGHTBUTTON,
                                   m_ptCursor.x, m_ptCursor.y, m_pWndParent );

            return ME_PROCESSED;    // Result will return with WM_COMMAND processing

    }


    return 0;
}


