// Pattern.cpp: implementation of the CPattern class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Vectal4.h"
#include "Pattern.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


void CPattern::InitDefaultNew()
{
    float r = 0.01f * ( rand()%200 - 100 );

    m_nKneesN     = 2;
    m_bDirs[0]    = DIR_REVERSE;
    m_bDirs[1]    = 0;
    m_pfPoinfs[0] = CPoinf(-100.0f,              000.0f);
    m_pfPoinfs[1] = CPoinf( 000.0f + 20.0f * r, -100.0f);
    m_pfPoinfs[2] = CPoinf( 100.0f,              000.0f);

    DetKneesC();
}

void CPattern::DetKneesC()
{
    for( int i=0; i<m_nKneesN; i++ )  {
        m_pfKneesC[i] = m_pfPoinfs[i] + m_pfPoinfs[i+1];
        m_pfKneesC[i] *= 0.5f;
    }
}




//        bMode:    0 - Add (in Knee), 1 - Select Point, 2 - Select Knee.
// return nElement: -1,0,..

int CPattern::DetCursorMode( const CPoinf& pf, float fScale, BYTE& bMode ) 
{    
    CPoinf d;
    float r;
    int i;

    int   nElem;
    float rMin = 99999999.0f;


    bMode = MODEADD;

    // Find Nearest Knee

    for( i=0; i<m_nKneesN; i++ )
    {
        d = m_pfKneesC[i] - pf;
        r = d.x*d.x + d.y*d.y;
        if( r < rMin )  {
            rMin = r;
            nElem = i;
        }        
    }

    // Maybe Have to Add Point to Begin or End

    d = m_pfPoinfs[0] - pf;             // Before First Point
    r = d.x*d.x + d.y*d.y;
    if( r < rMin )  {
        rMin = r;
        nElem = -1;
    }        
    
    d = m_pfPoinfs[m_nKneesN] - pf;     // After Last Point
    r = d.x*d.x + d.y*d.y;
    if( r < rMin )  {
        rMin = r;
        nElem = m_nKneesN;
    }


    // If rMin < Threshold, we have Selection, not Add

    float rThr = 20.0f / fScale;     // 20px - threshold
    rThr *= rThr;

    if( rMin < rThr )
    {
        if( -1 == nElem )  {            // Select First Point
            nElem = 0;              
            bMode = MODESELPOINT;
        }
        else if( m_nKneesN == nElem )   // Select Last Point
            bMode = MODESELPOINT;

        else                            // Select Knee
            bMode = MODESELKNEE;
    }
    else
        rMin = rThr;
    
    // Maybe We have to Select Other Point (Only if it's nearer than Threshold)


    for( i=1; i<m_nKneesN; i++ )
    {
        d = m_pfPoinfs[i] - pf;
        r = d.x*d.x + d.y*d.y;

        if( r < rMin )  {
            rMin = r;
            nElem = i;
            bMode = MODESELPOINT;
        }
    }


    return nElem;    
}




void CPattern::ChangeDir( int n, BYTE bBit )
{
    if( bBit )
        m_bDirs[n] ^= bBit;
    else  {        
        m_bDirs[n] += 1;
        m_bDirs[n] &= DIR_REVERSE | DIR_MIRROR;
    }
}


void CPattern::MovePoint( int n, const CPoinf& pf )
{
    m_pfPoinfs[n] = pf;

    DetKneesC();
}


int CPattern::AddPoint( int nAfter, const CPoinf& pf )
{
    if( MAXKNEES == m_nKneesN )
        return -1;

    m_bDirs[m_nKneesN] = m_bDirs[m_nKneesN-1];

    for( int i=m_nKneesN; i>nAfter; i-- )
    {
        m_pfPoinfs[i+1] = m_pfPoinfs[i];
        m_bDirs[i+1] = m_bDirs[i];
    }

    m_pfPoinfs[i+1] = pf;

    m_nKneesN ++;

    DetKneesC();

    return nAfter+1;
}


int CPattern::RemovePoint( int n )
{
    if( 2 == m_nKneesN )
        return -1;

    m_nKneesN --;

    for( int i=n; i<=m_nKneesN; i++ )
    {
        m_pfPoinfs[i] = m_pfPoinfs[i+1];
        m_bDirs   [i] = m_bDirs   [i+1];        
    }


    DetKneesC();

    if( n ) n --;
    return n;
    
}


void CPattern::SetDefaultView( const CPoinf& f0, float sc, const CPoinf& f1 )
{
    for( int i=0; i<=m_nKneesN; i++ )
    {
        m_pfPoinfs[i] -= f0;
        m_pfPoinfs[i] *= sc;
        m_pfPoinfs[i] += f1;
    }

    DetKneesC();
}



void CPattern::Serialize( CArchive& ar )
{
    int i;
    CString str;

	if( ar.IsStoring() )
	{
        str.Format( "VECTAL4\r\n" );        // Ascii Version
        ar.WriteString( (LPCTSTR)str );

        str.Format( "%d\r\n", m_nKneesN );
        ar.WriteString( (LPCTSTR)str );

        for( i=0; i<=m_nKneesN; i++ )
        {
            str.Format( "%f %f\r\n", m_pfPoinfs[i].x,
                                     m_pfPoinfs[i].y );
            ar.WriteString( (LPCTSTR)str );
        }

        for( i=0; i<m_nKneesN; i++ )
        {
            str.Format( "%d\r\n", m_bDirs[i] );
            ar.WriteString( (LPCTSTR)str );
        }

	}
	else
	{
        ar >> m_nKneesN;

        if( 0x54434556 != m_nKneesN )    // Old Hex Version
        {
            ar.Read ( m_pfPoinfs, (m_nKneesN+1) * sizeof(CPoinf) );
            ar.Read ( m_bDirs,     m_nKneesN    * sizeof(BYTE  ) );
        }
        else                            // "VECT" - Ascii Version
        {
            ar.ReadString( str ); // First String to End

            ar.ReadString( str );
            sscanf( (LPCTSTR)str, "%d", &m_nKneesN );

            for( i=0; i<=m_nKneesN; i++ )
            {
                ar.ReadString( str );
                sscanf( (LPCTSTR)str, "%f %f", &m_pfPoinfs[i].x, 
                                               &m_pfPoinfs[i].y );
            }

            for( i=0; i<m_nKneesN; i++ )
            {
                ar.ReadString( str );
                sscanf( (LPCTSTR)str, "%d", &m_bDirs[i] );
            }
        }

        DetKneesC();
	}
    
}
