// Spacer.cpp: implementation of the CSpacer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Vectal4.h"
#include "Spacer.h"

#include <math.h>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSpacer::CSpacer()
{
    Restore();

    m_ptOffset = CPoint( 0,0 );     // ? Centrize View Later

    m_bVerticalFlip = 0;
}


//////////////////////////////////////////////////////////////////////

void CSpacer::Restore()
{
    m_pfOrigin = CPoinf( 0.0f, 0.0f );
    m_fScale   = 1.0f;
}


CPoint CSpacer::ToPoint( CPoinf pf )
{
    pf -= m_pfOrigin;
    pf *= m_fScale;

    CPoint pt( (int)( pf.x + 0.5f),
               (int)(-pf.y + 0.5f) );

    if( m_bVerticalFlip )
        pt.y = -pt.y;

    pt += m_ptOffset;

    return pt;
}

CPoinf CSpacer::ToPoinf( CPoint pt )
{
    pt -= m_ptOffset;

    if( m_bVerticalFlip )
        pt.y = -pt.y;

    CPoinf pf( (float) pt.x,
               (float)-pt.y );

    pf *= 1.0f/m_fScale;
    pf += m_pfOrigin;

    return pf;
}




void CSpacer::MoveOrigin( const CPoint& pt, BYTE bButtonDown )
{
    static CPoinf pfOriginStart;
    static CPoinf pfCursorStart;

    if( bButtonDown )
    {   
        pfOriginStart = m_pfOrigin;     // Save StartOrigin & StartDrag
        pfCursorStart = ToPoinf( pt );
    }
    else
    {
        m_pfOrigin = pfOriginStart;

        CPoinf pf = ToPoinf( pt );

        pf -= pfCursorStart;
        m_pfOrigin -= pf;
    }
    
}



void CSpacer::Scale( CPoinf pfOriginMove, float d )
{
    float e  = expf( 0.1f * d );
    float e1 = 1.0f / e;


    m_fScale     *= e;                // Scale

    m_pfOrigin   *=      e1;          // Move Origin
    pfOriginMove *= 1.0f-e1;
    m_pfOrigin   += pfOriginMove;
}


void CSpacer::SetViewAsDefault( CPoinf* pf, int n )
{
    CPoinf f0,f1;
    float  sc;
    f0  = CSpacer::GetOrigin();
    sc  = CSpacer::GetScale();
    Restore();
    sc /= CSpacer::GetScale();
    f1  = CSpacer::GetOrigin();

    for( int i=0; i<=n; i++ )  {
        pf[i] -= f0;
        pf[i] *= sc;
        pf[i] += f1;
    }    
}




void CSpacer::Serialize( CArchive& ar )
{
    CString str;

	if( ar.IsStoring() )
	{
        str.Format( "Origin: %f %f\r\n", m_pfOrigin.x,
                                         m_pfOrigin.y );
        ar.WriteString( (LPCTSTR)str );

        str.Format( "Scale: %f\r\n", m_fScale );
        ar.WriteString( (LPCTSTR)str );
	}
	else
	{
        ar.ReadString( str );
        sscanf( (LPCTSTR)str, "Origin: %f %f", &m_pfOrigin.x,
                                               &m_pfOrigin.y );

        ar.ReadString( str );
        sscanf( (LPCTSTR)str, "Scale: %f", &m_fScale );
	}    
}
