// Vectal4View.cpp : implementation of the CVectal4View class
//

#include "stdafx.h"
#include "Vectal4.h"

#include "Vectal4Doc.h"
#include "Vectal4View.h"

#include "FileExportDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVectal4View

IMPLEMENT_DYNCREATE(CVectal4View, CView)

BEGIN_MESSAGE_MAP(CVectal4View, CView)
	//{{AFX_MSG_MAP(CVectal4View)
	ON_WM_SIZE()
	ON_WM_HSCROLL()
    ON_COMMAND( ID_FILE_EXPORT, OnFileExport )
	//}}AFX_MSG_MAP
    ON_COMMAND_RANGE( ID_VIEW_SHOWPATTERN, ID_VIEW_FILLMODE, OnViewChangeModes )
    ON_COMMAND_RANGE( ID_VIEW_SETASDEFAULTVIEW, ID_VIEW_RESTOREDEFAULTVIEW, OnViewDefaultView )
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVectal4View construction/destruction

CVectal4View::CVectal4View()
{
    m_bInitOnce = 1;
    m_bOpened4  = 0;

    CMouser::SetParent( this );
}

CVectal4View::~CVectal4View()
{
}


void CVectal4View::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	
    if( m_bInitOnce )   // Initializing
    {
        m_bInitOnce = 0;

        // nIterations and nLinesLimit do not change with New Document
        CWnd* pBar = ((CMainFrame*)GetParentFrame())->GetIterationsBar();
        pBar->SetDlgItemInt( IDC_EDIT_ITERATIONS, CDraw::m_nIterations );
        pBar->SetDlgItemInt( IDC_EDIT_TIMELIMIT,  CDraw::m_nTimeLimit  );
        pBar->SetDlgItemInt( IDC_EDIT_DETAIL,     CDraw::m_nDetail     );

        // Show Modes do not change with New Document
        CheckMenuItem( ID_VIEW_SHOWPATTERN, CDraw::m_bShowPattern );
        CheckMenuItem( ID_VIEW_SHOWFRACTAL, CDraw::m_bShowFractal );
        CheckMenuItem( ID_VIEW_FILLMODE,    CDraw::m_bFillMode    );

        // Redraw Slider Range and Pos
        CSliderCtrl* slIter = (CSliderCtrl*)pBar->GetDlgItem( IDC_SLIDER_ITERATIONS );
        slIter->SetRange( 0, 20 );
        slIter->SetPos( CDraw::m_nIterations );

    }	

    // Open or New Document
    if( m_bOpened4 )    // Opened File is 4 Version with saved Iteration, Modes etc
    {
        m_bOpened4 = 0;

        // Update Iteration State
        CWnd* pBar = ((CMainFrame*)GetParentFrame())->GetIterationsBar();
        pBar->SetDlgItemInt( IDC_EDIT_ITERATIONS, CDraw::m_nIterations );

        // Show Modes do not change with New Document
        CheckMenuItem( ID_VIEW_SHOWPATTERN, CDraw::m_bShowPattern );
        CheckMenuItem( ID_VIEW_SHOWFRACTAL, CDraw::m_bShowFractal );
        CheckMenuItem( ID_VIEW_FILLMODE,    CDraw::m_bFillMode    );

        CSliderCtrl* slIter = (CSliderCtrl*)pBar->GetDlgItem( IDC_SLIDER_ITERATIONS );
        slIter->SetPos( CDraw::m_nIterations );
    }
    else
    {        
        CSpacer::Restore();     // Restore Poinf Space to Default
    }

    //
    CDraw::m_bRedraw = 1;
    Invalidate( FALSE );
	
}


/////////////////////////////////////////////////////////////////////////////
// CVectal4View drawing

void CVectal4View::OnDraw( CDC* pDC )
{
    CBitmapWindow::BeginPaint( pDC );

    if( CDraw::m_bRedraw )
        CDraw::DrawVectal( GetDocument()->GetPattern() );

    CBitmapWindow::EndPaint( pDC );

}

/////////////////////////////////////////////////////////////////////////////
// CVectal4View diagnostics

#ifdef _DEBUG
void CVectal4View::AssertValid() const
{
	CView::AssertValid();
}

void CVectal4View::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CVectal4Doc* CVectal4View::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CVectal4Doc)));
	return (CVectal4Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CVectal4View message handlers

void CVectal4View::OnSize( UINT nType, int cx, int cy )  
{
	CView::OnSize( nType, cx, cy );
	
	//
    if( SIZE_MINIMIZED != nType )
    	CDraw::OnSize( cx, cy );

}



LRESULT CVectal4View::WindowProc( UINT message, WPARAM wParam, LPARAM lParam ) 
{
    DWORD dwRes = 0;

    CPoint pt;
    CPattern* pPattern;


    switch( message )
    {
        case WM_RBUTTONDOWN:
        case WM_LBUTTONDOWN:
        case WM_MBUTTONDOWN:
            wParam |= MK_BUTTONDOWN;    // Contain message to wParam

            pt.x = LOWORD(lParam);
            pt.y = HIWORD(lParam);

            if( WM_MBUTTONDOWN == message )
            {
                CMouser::SetCursorPoint( pt );  // Save only POINT
                dwRes = CMouser::OnMouseViewEvent( message, wParam );
            }

            else if( WM_LBUTTONDOWN == message )
            {
                pPattern = GetDocument()->GetPattern();
//                CMouser::SetCursorPoinf( pt );    // It's Done with WM_MOUSEMOVE.. or After PopupMenuLoop We have changed Arrow with WM_SETCURSOR processing
//                CMouser::TraceCursor( pPattern );
                dwRes = CMouser::OnMouseDocEvent( message, wParam, pPattern );

                if( ME_REDRAW & dwRes )
                {
                    CMouser::TraceCursor( pPattern );   // Trace Again
                    CMouser::DetArrow();
                    SetCursor( CMouser::m_hArrows[ CMouser::m_bArrow ] );                    
                }
            }

            else    // WM_RBUTTONDOWN
            {
                CMouser::SetCursorPoinf( pt );
                CMouser::SetCursorPoint( pt );  // Save POINT for PopupMenu

                pPattern = GetDocument()->GetPattern();
                CMouser::TraceCursor( pPattern );
                dwRes = CMouser::OnMouseDocEvent( message, wParam, pPattern );
            }

            break;
            

        case WM_MOUSEMOVE:

            pt.x = LOWORD(lParam);
            pt.y = HIWORD(lParam);

            if( MK_MBUTTON & wParam )
            {
                CMouser::SetCursorPoint( pt );  // Save only POINT - need converting to POINF inside CSpacer::MoveOrigin()
                dwRes = CMouser::OnMouseViewEvent( message, wParam );
            }
            else
            {
                pPattern = GetDocument()->GetPattern();

                CMouser::SetCursorPoinf( pt );   

                if( MK_LBUTTON & wParam )
                    // Dragging in prorgress - not Trace for Other Mode
                    dwRes = CMouser::OnMouseDocEvent( message, wParam, pPattern );

                else
                {
                    CMouser::TraceCursor( pPattern );
                    dwRes = ME_PROCESSED;
                }

                ChangeCursorStatus();   // at StatusBar
            }

            break;

//        case WM_LBUTTONUP:
//        case WM_LBUTTONDBLCLK:
        case WM_MOUSEWHEEL:

            pt.x = LOWORD(lParam);
            pt.y = HIWORD(lParam);
            ScreenToClient( &pt );

            CMouser::SetCursorPoinf( pt );
            dwRes = CMouser::OnMouseViewEvent( message, wParam );

            break;


        //--------------------------- Popup Menu

        case WM_COMMAND:
            if( 0 == HIWORD(wParam) )   // Menu
                switch( LOWORD(wParam) )
                {
                    case ID_MOUSER_REVERSE:
                    case ID_MOUSER_MIRROR:
                    case ID_MOUSER_HIDE:
                    case ID_MOUSER_STATIC:
                    case ID_MOUSER_REMOVE:
                    case ID_MOUSER_ADDPOINT:
                    case ID_POINT_COORDINATE:
                    case ID_VIEW_SETASDEFAULTVIEW:

                        pPattern = GetDocument()->GetPattern();
        //                CMouser::SetCursorPoinf( pt );    // It's Done with WM_LBUTTONDOWN
        //                CMouser::TraceCursor( pPattern );
                        dwRes = CMouser::OnMouseDocEvent( message, wParam, pPattern );

//                        if( ME_REDRAW | dwRes )  {
//                            CMouser::TraceCursor( pPattern );  After PopupMenuLoop We have changed Arrow with WM_SETCURSOR processing
//                            CMouser::DetArrow();
//                            SetCursor( CMouser::m_hArrows[ CMouser::m_bArrow ] );                    
//                        }
                        break;

                    case ID_VIEW_RESTOREDEFAULTVIEW:
                        dwRes = CMouser::OnMouseViewEvent( message, wParam );
                }

            break;

        //--------------------------- Cursor

        case WM_SETCURSOR:

            CMouser::m_nHitTest = LOWORD(lParam);   // Save nHitTest for WM_KEYDOWN and WM_KEYUP

            if( HTCLIENT == CMouser::m_nHitTest )
            {
                if( CMouser::m_bPopupMenuLoop )  {
                    CMouser::m_bPopupMenuLoop = 0;

                    GetCursorPos( &pt );
                    ScreenToClient( &pt );
                    CMouser::SetCursorPoinf( pt );

                    pPattern = GetDocument()->GetPattern();
                    CMouser::TraceCursor( pPattern );  // Trace if it was PopupMenu Loop

                    ChangeCursorStatus();
                }
                CMouser::DetArrow();
                SetCursor( CMouser::m_hArrows[ CMouser::m_bArrow ] );
                return TRUE;
            }

            break; // Do DefWindowProc


        case WM_KEYDOWN:
        case WM_KEYUP:
            if( VK_CONTROL == wParam )
            {
                CMouser::m_bCtrl = WM_KEYDOWN==message;

                if( HTCLIENT == CMouser::m_nHitTest )  {
                    CMouser::DetArrow();
                    SetCursor( CMouser::m_hArrows[ CMouser::m_bArrow ] );
                }

                return 0;
            }
            break;

    }

    
    if( !dwRes )
        return CView::WindowProc( message, wParam, lParam );
                

    if( ME_REDRAW & dwRes )
    {
        CDraw::m_bRedraw = 1;
        Invalidate( FALSE );
    }

    return 0;
}



void CVectal4View::OnHScroll( UINT nSBCode, UINT nPos, CScrollBar* pScrollBar ) 
{
    int nIter = ((CSliderCtrl*)pScrollBar)->GetPos();

    if( CDraw::m_nIterations != nIter )
    {   
        CDraw::m_nIterations = nIter;

        CWnd* pBar = ((CMainFrame*)GetParentFrame())->GetIterationsBar();
        pBar->SetDlgItemInt( IDC_EDIT_ITERATIONS, nIter );

        CDraw::m_bRedraw = 1;
        Invalidate( FALSE );
    }
	
	CView::OnHScroll(nSBCode, nPos, pScrollBar);
}



BOOL CVectal4View::OnCommand( WPARAM wParam, LPARAM lParam ) 
{
    WORD wNotifyCode = HIWORD(wParam);
    WORD wID         = LOWORD(wParam);


    // We Get Commands from IterationsBar

    if( EN_KILLFOCUS == wNotifyCode ||
        EN_CHANGE    == wNotifyCode )
    {   
        int* pValue = NULL;
        int  nLimit;

        switch( wID )
        {
            case IDC_EDIT_TIMELIMIT:
                pValue = &(CDraw::m_nTimeLimit);
                nLimit = TIMELIMITMAX;
                break;

            case IDC_EDIT_DETAIL:
                pValue = &(CDraw::m_nDetail);
                nLimit = DETAILMAX;
                break;

            case IDC_EDIT_ITERATIONS:
                pValue = &(CDraw::m_nIterations);
                nLimit = ITERATIONSMAX;
                break;
        }

        if( pValue )
        {
            CWnd* pWnd = CWnd::FromHandle((HWND)lParam)->GetParent(); 

            switch( HIWORD(wParam) )
            {
                case EN_KILLFOCUS:  // Restore Valid Value if EN_CHANGE do not changed
                    pWnd->SetDlgItemInt( wID, *pValue );
                    return TRUE;

                case EN_CHANGE:
                    int n = pWnd->GetDlgItemInt( wID );
                    if( n > 0  &&  n != *pValue  &&  n < nLimit )
                    {
                       *pValue = n;
                        CDraw::m_bRedraw = 1;
                        Invalidate( FALSE );

                        if( IDC_EDIT_ITERATIONS == wID )  {
                            CWnd* pBar = ((CMainFrame*)GetParentFrame())->GetIterationsBar();
                            CSliderCtrl* slIter = (CSliderCtrl*)pBar->GetDlgItem( IDC_SLIDER_ITERATIONS );
                            slIter->SetPos( n );
                        }
                    }
                    break;
            }

            return TRUE;

        }

    }
	
	return CView::OnCommand( wParam,lParam );
}


void CVectal4View::OnViewDefaultView( UINT nID ) 
{
    // I don't know how to do this otherwise,
    // but I want to process it in WindowProc()
    WindowProc( WM_COMMAND, nID, 0 );
}


void CVectal4View::OnViewChangeModes( UINT nID ) 
{
    BYTE bCheck = 2;

    switch( nID ){
        case ID_VIEW_SHOWPATTERN:  bCheck = CDraw::m_bShowPattern ^= 1;  break;
        case ID_VIEW_SHOWFRACTAL:  bCheck = CDraw::m_bShowFractal ^= 1;  break;
        case ID_VIEW_FILLMODE:     bCheck = CDraw::m_bFillMode    ^= 1;  break;
    }

    if( 2 ^ bCheck )    // Something Changed
    {
        CheckMenuItem( nID, bCheck );
        CDraw::m_bRedraw = 1;
        Invalidate( FALSE );
    }
	
}


void CVectal4View::ChangeCursorStatus()
{
    TCHAR str[0x100];
    CMouser::GetCursorStatus( str );

    ((CMainFrame*)GetParentFrame())->GetStatusBar()->SetPaneText( 1, str );
}


void CVectal4View::Serialize( CArchive& ar )
{
    CString str;

	if( ar.IsStoring() )
	{
        str.Format( "Iterations: %d\r\n", CDraw::m_nIterations );
        ar.WriteString( (LPCTSTR)str );

        str.Format( "Pattern: %d\r\n", CDraw::m_bShowPattern );
        ar.WriteString( (LPCTSTR)str );

        str.Format( "Fractal: %d\r\n", CDraw::m_bShowFractal );
        ar.WriteString( (LPCTSTR)str );

        str.Format( "Fill: %d\r\n", CDraw::m_bFillMode );
        ar.WriteString( (LPCTSTR)str );
	}
	else
	{
        int n;

        if( !ar.ReadString( str ) )  return;
        sscanf( (LPCTSTR)str, "Iterations: %d", &n );
        CDraw::m_nIterations = n;

        ar.ReadString( str );
        sscanf( (LPCTSTR)str, "Pattern: %d", &n );
        CDraw::m_bShowPattern = n;

        ar.ReadString( str );
        sscanf( (LPCTSTR)str, "Fractal: %d", &n );
        CDraw::m_bShowFractal = n;

        ar.ReadString( str );
        sscanf( (LPCTSTR)str, "Fill: %d", &n );
        CDraw::m_bFillMode = n;

        m_bOpened4 = 1;
	}

    CSpacer::Serialize( ar );
}



void CVectal4View::OnFileExport() 
{
    BOOL bSaved = 0;

    // Get File Name
    CFileExportDlg fdSave( FALSE, NULL, NULL,
                           OFN_EXPLORER | OFN_HIDEREADONLY |
                           OFN_OVERWRITEPROMPT | OFN_ENABLESIZING,
                          "Bitmap (*.bmp)|*.bmp|Adobe Illustrator (*.ai)|*.ai|",
                           this );

    fdSave.m_ofn.lpstrTitle = "Export";

    // Link ExportSettings
    ExportSettings es;
    fdSave.SetExportSettings( &es );
    // [In]
    CPattern* pPattern = GetDocument()->GetPattern();
    es.nKneesN = pPattern->GetKneesN(); 
    // [Out] - Set default values
    es.nWidth  = CBitmapWindow::m_nWindowW;
    es.nHeigth = CBitmapWindow::m_nWindowH;
    es.nIterations = CDraw::m_nIterations;

    // Run Dialog
    if( IDOK == fdSave.DoModal() )
    {
        int nType = fdSave.m_ofn.nFilterIndex;

        CString sFileName = fdSave.GetFileName();
        CString sFileExt  = fdSave.GetFileExt();

        // Add Extention if empty
        if( sFileExt.IsEmpty() )            
            sFileName += 1==nType ? ".bmp": ".ai";


        // Save Old Values
        int   nOldW = CBitmapWindow::m_nWindowW;
        int   nOldH = CBitmapWindow::m_nWindowH;
        int   nOldIters = CDraw::m_nIterations;
        float fOldScale = CSpacer::GetScale(); 

        // Prepare New View
        CDraw::OnSize( es.nWidth, es.nHeigth );
        CDraw::m_nIterations = es.nIterations;
        CDraw::m_bEconomy = 0;  // Full Draw
        CSpacer::SetScale( fOldScale * CBitmapWindow::m_nWindowW / nOldW );


        switch( nType )
        {
            case 2: // Ai
                bSaved = CDraw::DrawToAI( pPattern, (LPCTSTR)sFileName );
                break;

            case 1: // Bmp

                CBitmapWindow::m_bReInit = 2;   // Set Exact Size of Memory Bitmap
                CSpacer::SetVerticalFlip( 1 );

                // Redraw Vectal
                CDC* pDC = GetDC();
                 CBitmapWindow::BeginPaint( pDC );
                  CDraw::DrawVectal( pPattern );
                 CBitmapWindow::EndPaint( NULL );
                ReleaseDC(pDC);

                // Save BMP
                bSaved = CBitmapWindow::SaveAsBmp( (LPCTSTR)sFileName );

                CBitmapWindow::m_bReInit = 1;   // ReInit for Screen Size of Memory Bitmap
                CSpacer::SetVerticalFlip( 0 );
        }


        // Restore Old View
        CDraw::OnSize( nOldW, nOldH );
        CDraw::m_nIterations = nOldIters;
        CDraw::m_bEconomy = 1;
        CSpacer::SetScale( fOldScale );

    }
    
    // Now Set "Ready"
    ((CMainFrame*)GetParentFrame())->SetMessageText( bSaved? "Exported":"Cancel" );

}

